// Throttle function to limit how often a function is called
function throttle(func, limit) {
    let lastFunc;
    let lastRan;
    return function () {
        const context = this;
        const args = arguments;
        if (!lastRan) {
            func.apply(context, args);
            lastRan = Date.now();
        } else {
            clearTimeout(lastFunc);
            lastFunc = setTimeout(function () {
                if (Date.now() - lastRan >= limit) {
                    func.apply(context, args);
                    lastRan = Date.now();
                }
            }, limit - (Date.now() - lastRan));
        }
    };
}

// Function to create and insert the HintTube button
function insertHintTubeButton() {
    if (document.getElementById('hintTubeButton')) {
        positionHintTubeButton();
        return;
    }

    // Find the subscribe button
    const subscribeButton = document.querySelector('ytd-subscribe-button-renderer');

    if (subscribeButton) {
        // Create the HintTube button
        const hintTubeButton = document.createElement('button');
        hintTubeButton.id = 'hintTubeButton';
        hintTubeButton.textContent = 'HintTube';

        // Set the button CSS
        hintTubeButton.style.cssText = `
            position: absolute;
            cursor: pointer;
            padding: 10px 20px;
            border-radius: 20px;
            border: none;
            background-color: rgba(0, 0, 0, 0.05);
            color: #35489e;
            font-size: 14px;
            font-weight: bold;
            z-index: 1000;
        `;

        // Append the button to the same parent as the subscribe button
        const parentContainer = subscribeButton.parentElement;
        parentContainer.style.position = 'relative'; // Ensure parent is positioned
        parentContainer.appendChild(hintTubeButton);

        // Cache the subscribe button's position
        updateSubscribeButtonRect();

        // Position the button initially
        positionHintTubeButton();

        // Add click event listener to the button
        hintTubeButton.addEventListener('click', function () {
            const currentUrl = window.location.href;
            const hintTubeUrl = currentUrl.replace('youtube.com', 'hinttube.com');
            window.open(hintTubeUrl, '_blank');
        });

        console.log('HintTube button inserted.');
    } else {
        console.log('Subscribe button not found.');
    }
}

// Cached position of the subscribe button
let subscribeButtonRect = null;

// Function to update the cached position of the subscribe button
function updateSubscribeButtonRect() {
    const subscribeButton = document.querySelector('ytd-subscribe-button-renderer');
    if (subscribeButton) {
        subscribeButtonRect = subscribeButton.getBoundingClientRect();
    }
}

// Function to position the HintTube button
function positionHintTubeButton() {
    const hintTubeButton = document.getElementById('hintTubeButton');
    const subscribeButton = document.querySelector('ytd-subscribe-button-renderer');

    if (hintTubeButton && subscribeButton) {
        // Position the HintTube button relative to the subscribe button
        hintTubeButton.style.top = `${subscribeButton.offsetTop}px`;
        hintTubeButton.style.left = `${subscribeButton.offsetLeft + subscribeButton.offsetWidth + 10}px`;
    }
}

// Function to initialize the MutationObserver and insert the button
function initializeHintTubeButton() {
    // Create a MutationObserver to detect changes in the DOM
    const observer = new MutationObserver(function () {
        insertHintTubeButton();
    });

    // Start observing the body for changes
    observer.observe(document.body, { childList: true, subtree: true });

    // Insert the button initially (in case the subscribe button is already available)
    insertHintTubeButton();
}

// Throttle the position updates on scroll and resize
const throttledPositionHintTubeButton = throttle(positionHintTubeButton, 100);
window.addEventListener('scroll', throttledPositionHintTubeButton);
window.addEventListener('resize', () => {
    updateSubscribeButtonRect();
    throttledPositionHintTubeButton();
});

// Initialize the script
initializeHintTubeButton();
